package com.compo.camera.activities.main;

import android.app.Activity;
import android.content.Intent;
import android.media.MediaPlayer;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.provider.MediaStore;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.Toast;

import com.compo.inspector.R;

import java.io.File;

public class MainActivity extends Activity
{
	protected static final int ACTION_TAKE_PHOTO = 2;
	protected static final String BITMAP_STORAGE_KEY = "storage_bitmap";

	protected ImageView view;
	protected Uri imageUri;

	protected boolean exit;

	private MediaPlayer media;

	/**
	 * Called when the activity is first created.
	 */
	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);

		this.requestWindowFeature(Window.FEATURE_NO_TITLE);
		this.getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
		this.getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN | WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON, WindowManager.LayoutParams.FLAG_FULLSCREEN);

		this.setContentView(R.layout.main);

		this.playSound(R.raw.start_jingle, false);

		this.view = (ImageView) findViewById(R.id.image);
		this.view.setPadding(25, 0, 25, 0);

		this.exit = false;
	}

	protected void playSound(int id, boolean loop)
	{
		try {
			if (null == this.media) {
				this.media = MediaPlayer.create(this, id);
			} else if (this.media.isPlaying()) {
				this.media.stop();
				this.media.release();
			}

			this.media = MediaPlayer.create(this, id);
			this.media.start();
		} catch (Exception e) {
			Log.e(MainActivity.class.toString(), "Media player error for '" + id + "' sound!");
		}
	}

	protected void dispatchPictureIntent(int code)
	{
		File photo = new File(Environment.getExternalStorageDirectory(), "picture");

		this.imageUri = Uri.fromFile(photo);

		Intent intent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
		intent.putExtra(MediaStore.EXTRA_OUTPUT, this.imageUri);

		this.startActivityForResult(intent, code);
	}

	protected void handleSmallCameraPhoto(Intent intent)
	{
		this.view.setImageURI(null);
		this.view.setPadding(0, 0, 0, 0);
		this.view.setImageURI(this.imageUri);
		this.view.setVisibility(View.VISIBLE);
	}

	@Override
	public void onBackPressed()
	{
		if (this.exit) {
			this.finish();
		} else {
			Toast.makeText(this, "Naciśnij ponownie, aby wyjść ...", Toast.LENGTH_SHORT).show();
			this.exit = true;

			new Handler().postDelayed(new Runnable()
			{
				@Override
				public void run()
				{
					exit = false;
				}
			}, 3 * 1000);
		}
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data)
	{
		switch (requestCode) {
			case ACTION_TAKE_PHOTO: {
				if (RESULT_OK == resultCode) {
					this.handleSmallCameraPhoto(data);
				}
				break;
			}
		}
	}

	@Override
	protected void onSaveInstanceState(Bundle outState)
	{
		outState.putParcelable(BITMAP_STORAGE_KEY, this.imageUri);

		super.onSaveInstanceState(outState);
	}

	@Override
	protected void onRestoreInstanceState(Bundle savedInstanceState)
	{
		super.onRestoreInstanceState(savedInstanceState);

		this.imageUri = savedInstanceState.getParcelable(BITMAP_STORAGE_KEY);

		this.view.setImageURI(this.imageUri);
		this.view.setVisibility(ImageView.VISIBLE);
	}
}