package com.compo.camera.view;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.PorterDuff;
import android.view.MotionEvent;
import android.view.View;

/**
 * Created by lmodlinski on 25/03/17.
 */
public class DrawingView extends View
{
	private Context context;
	private Paint contextPaint;

	public int width;
	public int height;

	private boolean locked;

	private Bitmap bitmap;
	private Canvas canvas;
	private Path path;
	private Paint paint;

	private Paint circlePaint;
	private Path circlePath;

	private float mX, mY;
	private static final float TOUCH_TOLERANCE = 4;

	public DrawingView(Context context, Paint contextPaint)
	{
		super(context);

		this.context = context;
		this.contextPaint = contextPaint;

		this.locked = false;

		this.path = new Path();
		this.paint = new Paint(Paint.DITHER_FLAG);

		this.circlePath = new Path();

		this.circlePaint = new Paint();
		this.circlePaint.setAntiAlias(true);
		this.circlePaint.setColor(Color.BLUE);
		this.circlePaint.setStyle(Paint.Style.STROKE);
		this.circlePaint.setStrokeJoin(Paint.Join.MITER);
		this.circlePaint.setStrokeWidth(4f);
	}

	@Override
	protected void onSizeChanged(int w, int h, int oldw, int oldh)
	{
		super.onSizeChanged(w, h, oldw, oldh);

		this.bitmap = Bitmap.createBitmap(w, h, Bitmap.Config.ARGB_8888);
		this.canvas = new Canvas(this.bitmap);
	}

	@Override
	protected void onDraw(Canvas canvas)
	{
		super.onDraw(canvas);

		canvas.drawBitmap(this.bitmap, 0, 0, this.paint);
		canvas.drawPath(this.path, this.contextPaint);
		canvas.drawPath(this.circlePath, this.circlePaint);
	}

	public void setContextPaint(Paint paint)
	{
		this.contextPaint = paint;
	}

	public void clear()
	{
		this.canvas.drawColor(0, PorterDuff.Mode.CLEAR);
		this.invalidate();
	}

	private void touch_start(float x, float y)
	{
		this.path.reset();
		this.path.moveTo(x, y);

		this.mX = x;
		this.mY = y;
	}

	private void touch_move(float x, float y)
	{
		float dx = Math.abs(x - this.mX);
		float dy = Math.abs(y - this.mY);
		if (dx >= TOUCH_TOLERANCE || dy >= TOUCH_TOLERANCE) {
			this.path.quadTo(this.mX, this.mY, (x + this.mX) / 2, (y + this.mY) / 2);
			this.mX = x;
			this.mY = y;

			this.circlePath.reset();
			this.circlePath.addCircle(this.mX, this.mY, 30, Path.Direction.CW);
		}
	}

	private void touch_up()
	{
		this.path.lineTo(this.mX, this.mY);

		this.circlePath.reset();
		this.canvas.drawPath(this.path, this.contextPaint);
		this.path.reset();

	}

	public void lock()
	{
		this.locked = true;
	}

	public void unlock()
	{
		this.locked = false;
	}

	@Override
	public boolean onTouchEvent(MotionEvent event)
	{
		if (!this.locked) {
			float x = event.getX();
			float y = event.getY();

			switch (event.getAction()) {
				case MotionEvent.ACTION_DOWN:
					this.touch_start(x, y);
					this.invalidate();
					break;
				case MotionEvent.ACTION_MOVE:
					this.touch_move(x, y);
					this.invalidate();
					break;
				case MotionEvent.ACTION_UP:
					this.touch_up();
					this.invalidate();
					break;
			}
		}

		return true;
	}
}
