package com.compo.inspector.activities.main;

import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.GradientDrawable;
import android.os.Bundle;
import android.provider.MediaStore;
import android.util.TypedValue;
import android.view.View;
import android.widget.Button;
import android.widget.GridLayout;
import android.widget.ImageView;

import com.compo.camera.activities.main.MainActivity;
import com.compo.camera.algorithms.ColorAssessment;
import com.compo.inspector.R;

import java.io.IOException;
import java.util.Random;

public class InspectorMainActivity extends MainActivity
{
	protected static final double INITIAL_THRESHOLD = 85;
	protected static final double INCR_THRESHOLD = 13;
	protected static final double MAX_THRESHOLD = 150.0;

	protected static final double POPULATION = 0.01;
	protected static final int PALLETE = 32;

	protected int color;
	protected double threshold;
	protected boolean result;

	private static final int[] COLORS = {
			Color.rgb(132, 205, 71),
			Color.rgb(251, 154, 77),
			Color.rgb(52, 63, 117),
			Color.rgb(230, 230, 230),
			Color.rgb(186, 15, 211),
			Color.rgb(34, 135, 67),
			Color.rgb(58, 116, 176),
			Color.rgb(211, 15, 39),
			Color.rgb(249, 223, 66),
			Color.rgb(38, 38, 46),
			Color.rgb(204, 148, 50),
	};

	Button.OnClickListener onCameraClickListener =
			new Button.OnClickListener()
			{
				@Override
				public void onClick(View v)
				{
					dispatchPictureIntent(ACTION_TAKE_PHOTO);

					playSound(R.raw.click, false);
				}
			};

	Button.OnClickListener onAgainClickListener =
			new Button.OnClickListener()
			{
				@Override
				public void onClick(View v)
				{
					again();

					playSound(R.raw.click, false);
				}
			};

	Button.OnClickListener onNextClickListener =
			new Button.OnClickListener()
			{
				@Override
				public void onClick(View v)
				{
					next();

					playSound(R.raw.click, false);
				}
			};

	private void again()
	{
		this.change(true);
		this.view.setImageURI(null);

		if (!this.result) {
			this.threshold = Math.min(Math.max(this.threshold + INCR_THRESHOLD, INITIAL_THRESHOLD), MAX_THRESHOLD);
		}
	}

	private void next()
	{
		this.change(true);
		this.randomizeColor();

		this.threshold = INITIAL_THRESHOLD;
		this.view.setImageURI(null);
	}

	@Override
	protected void handleSmallCameraPhoto(Intent intent)
	{
		super.handleSmallCameraPhoto(intent);

		try {
			if (this.result = ColorAssessment.assess(
					MediaStore.Images.Media.getBitmap(this.getContentResolver(), this.imageUri),
					this.threshold,
					this.color,
					PALLETE,
					POPULATION
			)) {
				((ImageView) this.findViewById(R.id.text)).setImageResource(R.drawable.result_yes);

				playSound(R.raw.good_answer, false);
			} else {
				((ImageView) this.findViewById(R.id.text)).setImageResource(R.drawable.result_no);

				playSound(R.raw.wrong_answer, false);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}

		this.change(false);
	}

	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);

		this.threshold = INITIAL_THRESHOLD;

		this.randomizeColor();
		this.change(true);

		((ImageView) this.findViewById(R.id.enc1)).setImageResource(R.drawable.inspect_look_for);
		((ImageView) this.findViewById(R.id.loop)).setImageResource(R.drawable.icon_binocular);
		((ImageView) this.findViewById(R.id.enc2)).setImageResource(R.drawable.inspect_do);
		((Button) this.findViewById(R.id.camera)).setBackgroundResource(R.drawable.button_camera);

		((Button) findViewById(R.id.camera)).setOnClickListener(this.onCameraClickListener);
		((Button) findViewById(R.id.again)).setOnClickListener(this.onAgainClickListener);
		((Button) findViewById(R.id.next)).setOnClickListener(this.onNextClickListener);
	}

	private void change(boolean main)
	{
		((GridLayout) findViewById(R.id.taking)).setVisibility(main ? View.VISIBLE : View.INVISIBLE);
		((GridLayout) findViewById(R.id.result)).setVisibility(main ? View.INVISIBLE : View.VISIBLE);
		((GridLayout) findViewById(R.id.color)).setVisibility(main ? View.INVISIBLE : View.VISIBLE);
	}

	private void randomizeColor()
	{
		this.color = COLORS[new Random().nextInt(COLORS.length)];

		((GridLayout) this.findViewById(R.id.color)).setBackgroundColor(this.color);

		this.fillColor();
	}

	private void fillColor()
	{
		Drawable currentDrawable = ((ImageView) this.findViewById(R.id.loop_color)).getDrawable();
		GradientDrawable colorChoiceDrawable;
		if (currentDrawable != null && currentDrawable instanceof GradientDrawable) {
			colorChoiceDrawable = (GradientDrawable) currentDrawable;
		} else {
			colorChoiceDrawable = new GradientDrawable();
		}

		// Set stroke to dark version of color
		colorChoiceDrawable.setColor(this.color);
		colorChoiceDrawable.setStroke((int) TypedValue.applyDimension(
				TypedValue.COMPLEX_UNIT_DIP, 1.0f, this.getResources().getDisplayMetrics()), this.color);

		((ImageView) this.findViewById(R.id.loop_color)).setImageDrawable(colorChoiceDrawable);
		((ImageView) this.findViewById(R.id.loop_color)).setVisibility(View.VISIBLE);
	}
}
